"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t;
    return { next: verb(0), "throw": verb(1), "return": verb(2) };
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var path_1 = require("path");
var url_1 = require("url");
var fs_1 = require("fs");
var crypto_1 = require("crypto");
var child_process_1 = require("child_process");
var semver = require("semver");
var tmp = require("tmp");
var debug = require('debug/src/browser')('nsis-compat-updater');
var got = require('got');
var progressStream = require('progress-stream');
var NsisCompatUpdater = (function () {
    function NsisCompatUpdater(seed, currentVersion, currentArch) {
        this.seed = seed;
        this.currentVersion = currentVersion;
        this.currentArch = currentArch;
    }
    NsisCompatUpdater.prototype.checkForUpdates = function () {
        return __awaiter(this, void 0, void 0, function () {
            var versionInfo;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        debug('in checkForUpdates');
                        return [4 /*yield*/, this.getVersionInfo()];
                    case 1:
                        versionInfo = _a.sent();
                        if (!semver.gt(versionInfo.latest, this.currentVersion)) {
                            return [2 /*return*/, null];
                        }
                        return [4 /*yield*/, this.getVersion(versionInfo.latest)];
                    case 2: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    NsisCompatUpdater.prototype.downloadUpdate = function (version) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            var _a, installers, updaters, _b, url, hash, path;
            return __generator(this, function (_c) {
                switch (_c.label) {
                    case 0:
                        debug('in downloadUpdate', 'version', version);
                        return [4 /*yield*/, this.getVersion(version)];
                    case 1:
                        _a = _c.sent(), installers = _a.installers, updaters = _a.updaters;
                        _b = (function () {
                            var updater = updaters.filter(function (updater) { return updater.fromVersion == _this.currentVersion && updater.arch == _this.currentArch; })[0];
                            if (updater) {
                                return {
                                    url: "" + url_1.resolve(path_1.dirname(_this.seed), updater.path),
                                    hash: updater.hash,
                                };
                            }
                            var installer = installers.filter(function (installer) { return installer.arch == _this.currentArch; })[0];
                            if (installer) {
                                return {
                                    url: "" + url_1.resolve(path_1.dirname(_this.seed), installer.path),
                                    hash: installer.hash,
                                };
                            }
                            throw new Error('ERROR_UPDATER_NOT_FOUND');
                        })(), url = _b.url, hash = _b.hash;
                        return [4 /*yield*/, this.tmpUpdateFile()];
                    case 2:
                        path = _c.sent();
                        debug('in downloadUpdate', 'url', url);
                        return [4 /*yield*/, this.download(url, path)];
                    case 3:
                        _c.sent();
                        debug('in downloadUpdate', 'path', path);
                        return [4 /*yield*/, this.checkFileHash('sha256', path, hash)];
                    case 4:
                        if (!(_c.sent())) {
                            throw new Error('ERROR_HASH_MISMATCH');
                        }
                        return [2 /*return*/, path];
                }
            });
        });
    };
    NsisCompatUpdater.prototype.install = function (path, slient) {
        if (slient === void 0) { slient = false; }
        debug('in install', 'path', path);
        debug("in install", 'slient', slient);
        var args = [];
        var options = {
            detached: true,
            stdio: 'ignore',
        };
        if (slient) {
            args.push('/S');
        }
        try {
            child_process_1.spawn(path, args, options)
                .unref();
        }
        catch (err) {
            if (err.code == 'UNKNOWN') {
                /*
                // TODO: Elevate and run again.

                spawn(elevate, [ path, ...args ], options)
                .unref();
                */
            }
            else {
                throw err;
            }
        }
    };
    NsisCompatUpdater.prototype.installWhenQuit = function (path) {
        var _this = this;
        console.info('installWhenQuit');
        if (process.versions.nw) {
            throw new Error('ERROR_UNKNOWN');
        }
        else if (process.versions.electron) {
            return require('electron').app.on('quit', function () { return _this.install(path, true); });
        }
        else {
            throw new Error('ERROR_UNKNOWN');
        }
    };
    NsisCompatUpdater.prototype.quitAndInstall = function (path) {
        console.info('quitAndInstall');
        if (process.versions.nw) {
            this.install(path, false);
            nw.App.quit();
        }
        else if (process.versions.electron) {
            return require('electron').app.quit();
        }
        else {
            throw new Error('ERROR_UNKNOWN');
        }
    };
    NsisCompatUpdater.prototype.getVersion = function (version) {
        return __awaiter(this, void 0, void 0, function () {
            var versionInfo, item;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getVersionInfo()];
                    case 1:
                        versionInfo = _a.sent();
                        item = versionInfo.versions.filter(function (item) { return item.version == version; })[0];
                        if (!item) {
                            throw new Error('ERROR_VERSION_NOT_FOUND');
                        }
                        return [2 /*return*/, item];
                }
            });
        });
    };
    NsisCompatUpdater.prototype.tmpUpdateFile = function () {
        return new Promise(function (resolve, reject) {
            tmp.file({
                postfix: '.exe',
                discardDescriptor: true,
            }, function (err, path, fd, cleanup) { return err ? reject(err) : resolve(path); });
        });
    };
    NsisCompatUpdater.prototype.checkFileHash = function (type, path, expected) {
        return new Promise(function (resolve, reject) {
            var hasher = crypto_1.createHash(type);
            hasher.on('error', reject);
            hasher.on('readable', function () {
                var data = hasher.read();
                if (data) {
                    resolve(data.toString('hex') == expected);
                }
            });
            fs_1.createReadStream(path).pipe(hasher);
        });
    };
    NsisCompatUpdater.prototype.getVersionInfo = function () {
        return __awaiter(this, void 0, void 0, function () {
            var versionInfo;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!!this.versionInfo) return [3 /*break*/, 2];
                        return [4 /*yield*/, got(this.seed, {
                                timeout: 5000,
                            })
                                .then(function (res) { return JSON.parse(res.body); })];
                    case 1:
                        versionInfo = _a.sent();
                        debug('in getVersionInfo', 'versionInfo', versionInfo);
                        this.versionInfo = versionInfo;
                        _a.label = 2;
                    case 2: return [2 /*return*/, this.versionInfo];
                }
            });
        });
    };
    NsisCompatUpdater.prototype.download = function (url, path, onProgress) {
        return __awaiter(this, void 0, void 0, function () {
            var stream, size, progress;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        stream = got.stream(url);
                        return [4 /*yield*/, new Promise(function (resolve, reject) {
                                stream.on('error', reject);
                                stream.on('response', resolve);
                            })
                                .then(function (res) { return res.headers['content-type']; })];
                    case 1:
                        size = _a.sent();
                        progress = progressStream({
                            length: size,
                            time: 1000,
                        });
                        progress.on('progress', onProgress ? onProgress : function (state) {
                            debug('in handleProgress', 'state.speed', state.speed);
                        });
                        return [4 /*yield*/, new Promise(function (resolve, reject) {
                                stream.pipe(progress)
                                    .pipe(fs_1.createWriteStream(path))
                                    .on('finish', resolve);
                            })];
                    case 2:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    return NsisCompatUpdater;
}());
exports.NsisCompatUpdater = NsisCompatUpdater;
//# sourceMappingURL=NsisCompatUpdater.js.map